@@ -132,12 +132,16 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      th0 <- z$th0
 #  handle points where estimation failed
      n <- prod(ddim)
-     indD <- (1:n)[D[2,,,]==0&D[3,,,]==0&D[5,,,]==0&mask]
+     dim(mask) <- c(1, ddim)
+     indD <- (1:n)[D[2,,,, drop=FALSE]==0&D[3,,,, drop=FALSE]==0&D[5,,,, drop=FALSE]==0&mask]
+     dim(mask) <- ddim
+# this does not work in case of 2D data: 
+#     indD <- (1:n)[D[2,,,]==0&D[3,,,]==0&D[5,,,]==0&mask]
      if(length(indD)>0){
      dim(si) <- c(ngrad,n)
      dim(D) <- c(6,n)
      dim(res) <- c(ngrad,n)
-     cat("length of IndD",length(indD),"\n")
+#     cat("length of IndD",length(indD),"\n")
      for(i in indD){
         zz <- optim(c(1,0,0,1,0,1),opttensR,method="BFGS",si=si[-s0ind,i],s0=s0[i],grad=grad[,-s0ind],sdcoef=sdcoef)
         D[,i] <- rho2D(zz$par)
