@@ -2,7 +2,7 @@ dwiMtImprove <- function( mtobj,dwiobj, ...) cat("No dwiMixtensor calculation de
 
 setGeneric("dwiMtImprove", function( mtobj,dwiobj, ...) standardGeneric("dwiMtImprove"))
 
-setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, maxcomp=3,  p=40, method="mixtensor", reltol=1e-6, maxit=5000,ngc=1000, optmethod="BFGS", nguess=100*maxcomp^2,msc="BIC",pen=NULL,where=NULL){
+setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, maxcomp=3, method="mixtensor", reltol=1e-6, maxit=5000,ngc=1000, optmethod="BFGS", nguess=100*maxcomp^2,msc="BIC",pen=NULL,where=NULL,new=FALSE){
 #
 #  uses  S(g)/s_0 = w_0 exp(-l_1) +\sum_{i} w_i exp(-l_2-(l_1-l_2)(g^T d_i)^2)
 #
@@ -14,6 +14,7 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
   args <- sys.call(-1)
   args <- c(mtobj@call,args)
   if(is.null(pen)) pen <- 100
+  if(method=="mixtensoriso") optmethod <- "L-BFGS-B"
   ngrad <- mtobj@ngrad
   ddim <- mtobj@ddim
   mask <- mtobj@mask
@@ -78,12 +79,14 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
 #
 #   compute penalty for model selection, default BIC
 #
-  penIC0 <- switch(msc,"AIC"=2*npar/ngrad0,"BIC"=log(ngrad0)*npar0/ngrad0,
+  penIC0 <- switch(msc,"AIC"=2*npar0/ngrad0,"BIC"=log(ngrad0)*npar0/ngrad0,
                   "AICC"=(1+npar0/ngrad0)/(1-(npar+2)/ngrad0),
+                  "None"=log(ngrad0)-log(ngrad0-npar0),
                   log(ngrad0)*npar0/ngrad0)
 # used to destinguish between initial estimates with qnd without isotropic part
   penIC <- switch(msc,"AIC"=2*npar/ngrad0,"BIC"=log(ngrad0)*npar/ngrad0,
                   "AICC"=(1+npar/ngrad0)/(1-(npar+2)/ngrad0),
+                  "None"=log(ngrad0)-log(ngrad0-npar),
                   log(ngrad0)*npar/ngrad0)
   cat("End generating auxiliary objects",format(Sys.time()),"\n")
 #
@@ -110,6 +113,7 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
      smix <- sum(mix[,i1,i2,i3])
 # used to destinguish between initial estimates with qnd without isotropic part
      krit <- log(sigma2[i1,i2,i3])+ if(smix>1-1e-8) penIC0[ordi+1] else penIC[ordi+1]
+     if(new) krit<-1e20
      if(where[i1,i2,i3]){ 
 #   only analyze voxel within mask
      mc0 <- maxcomp
@@ -128,7 +132,15 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
                      npar=integer(1),
                      DUPL=TRUE,
                      PACKAGE="dti")[c("param","npar")]   
-     par <- z$param
+     npar <- z$npar
+     if(any(is.na(z$param))||any(is.infinite(z$param))){
+         cat("voxel",i1,i2,i3,"param",z$param,"npar",npar,"\n")
+         npar <- length(!is.na(cumsum(z$param)))
+     }
+     param <- z$param[1:npar]
+     if(optmethod=="L-BFGS-B") param <- c(param,switch(method,
+                                        "mixtensor"=rep(1/npar,npar),
+                                        "mixtensoriso"=rep(1/npar,(npar+1))))
 #
 #   these are the gradient vectors corresponding to minima in spherical coordinates
 #
@@ -136,55 +148,60 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
 #  use AIC/ngrad0, BIC/ngrad0 or AICC/ngrad0 respectively
 #
 #        cat("i",i1,i2,i3,"par",par[1:z$npar],"npar",z$npar,"\n")
-     mc0 <- (z$npar+1)/2
+     mc0 <- (npar-1)/2
 
      for(k in mc0:1){ # begin model order
         if(k<ord) {
 #
 #  otherwise we would reanalyze a model
 #
-        if(method=="mixtensor"){
-           lpar <- 2*k+1
-#
-#        cat("par",par[1:(2*k+1)],"pen",pen,"krit",krit,"\n")
-           if(optmethod=="BFGS"){
-#                 z <- optim(par[1:(2*k+1)],mfunpl0,gmfunpl0,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-#                         method="BFGS",control=list(maxit=maxit,reltol=reltol))
-                 z <- optim(par[1:lpar],mfunpl0,gmfunpl0,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-                         method="BFGS",control=list(maxit=maxit,reltol=reltol))
-           } else {
-              z <- optim(par[1:lpar],mfunpl0,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-                         method=optmethod,control=list(maxit=maxit,reltol=reltol))
-           }
-        } else if(method=="mixtensoriso"){
-           lpar <- 2*k+1
-#
-           if(optmethod=="BFGS"){
-                 z <- optim(par[1:lpar],mfunpl1,gmfunpl1,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-                         method="BFGS",control=list(maxit=maxit,reltol=reltol))
-           } else {
-              z <- optim(par[1:lpar],mfunpl1,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-                         method=optmethod,control=list(maxit=maxit,reltol=reltol))
-           }
-        }         
-#        cat("opt-par",z$par,"value",z$value,"krit",krit,"\n")
-# thats sum of squared residuals + penalties (w<0 or 0>th or or th > 8)
-#
-#   estimate of sigma from the best fitting model
-#
-        if(method=="mixtensor"){
-            zz <- mfunplwghts0(z$par,siq[i1,i2,i3,],grad,pen)
-        } else if (method=="mixtensoriso"){
-            zz <- mfunplwghts1(z$par,siq[i1,i2,i3,],grad,pen)
+        if(optmethod=="L-BFGS-B"){
+        param <- if(method=="mixtensor") param[1:(3*k+1)] else param[1:(3*k+2)]
+        param[-(1:(2*k+1))] <- if(method=="mixtensor") rep(1/k,k) else rep(1/(k+1),k+1)
         }
-        value <- zz$value 
+        if(method=="mixtensor"){
+#
+           z <- switch(optmethod,
+                    "BFGS"=optim(param[1:(2*k+1)],mfunpl0,gmfunpl0,
+                           siq=siq[i1,i2,i3,],grad=grad,pen=pen,
+                           method="BFGS",control=list(maxit=maxit,reltol=reltol)),
+                    "CG"=optim(param[1:(2*k+1)],mfunpl0,gmfunpl0,
+                           siq=siq[i1,i2,i3,],grad=grad,pen=pen,
+                           method="CG",control=list(maxit=maxit,reltol=reltol)),
+                    "Nelder-Mead"=optim(param[1:(2*k+1)],mfunpl0h,
+                           siq=siq[i1,i2,i3,],grad=grad,method="Nelder-Mead",
+                           control=list(maxit=maxit,reltol=reltol)),
+                    "L-BFGS-B"=optim(param[1:(3*k+1)],mfunpl,gmfunpl,
+                           siq=siq[i1,i2,i3,],grad=grad,method="L-BFGS-B",
+                           lower=c(0,rep(-Inf,2*k),rep(0,k)),
+                           control=list(maxit=maxit,factr=reltol/.Machine$double.eps)))
+        } else { # method=="mixtensoriso"
+#           cat("voxel",i1,i2,i3,"param",param[1:(3*k+2)],"\n")
+           z <- optim(param[1:(3*k+2)],mfunpli,gmfunpli,
+                           siq=siq[i1,i2,i3,],grad=grad,method="L-BFGS-B",
+                           lower=c(0,rep(-Inf,2*k),rep(0,k+1)),
+                           control=list(maxit=maxit,factr=reltol/.Machine$double.eps))
+#
+#  other methods seem less numerically stable in this situation
+#
+        }        
+        if(method=="mixtensor"){
+            zz <- switch(optmethod,
+                    "BFGS"=mfunplwghts0(z$par[1:(2*k+1)],siq[i1,i2,i3,],grad,pen),
+                    "CG"=mfunplwghts0(z$par[1:(2*k+1)],siq[i1,i2,i3,],grad,pen),
+                    "Nelder-Mead"=mfunplwghts0h(z$par[1:(2*k+1)],siq[i1,i2,i3,],grad),
+                    "L-BFGS-B"=mfunwghts(z$par[1:(3*k+1)],siq[i1,i2,i3,],grad))
+        } else if (method=="mixtensoriso"){
+            zz <- mfunwghtsi(z$par[1:(3*k+2)],siq[i1,i2,i3,],grad)
+        }
+        value <- if(optmethod=="L-BFGS-B") z$value else zz$value
         ord <- zz$ord
 #  replace sigmai by best variance estimate from currently best model
         if(any(zz$lev<0)||ord<k){
            ttt <- krit
 #   parameters not interpretable reduce order
         } else {
-           si2new <- value/(ngrad0-3*ord-1)
+           si2new <- value/ngrad0 # changed 2011/05/13 
            if(si2new<1e-15){
                cat(i1,i2,i3,ord,si2new,"\n")
                si2new <- 1e-15
@@ -196,7 +213,7 @@ setMethod("dwiMtImprove",c("dwiMixtensor","dtiData"), function(mtobj, dwiobj, ma
 #
 #     use directions corresponding to largest weights as initial directions
 #
-        if(ttt < krit) {
+        if(ttt < krit & ord>0) {
            krit <- ttt
            norder[i1,i2,i3] <- ord
            nlev[,i1,i2,i3] <- zz$lev
@@ -305,9 +322,11 @@ setMethod("dwiMtCombine",c("dwiMixtensor","dwiMixtensor"), function(mtobj1,mtobj
 #
   penIC1 <- switch(msc,"AIC"=2*npar1/ngrad0,"BIC"=log(ngrad0)*npar1/ngrad0,
                   "AICC"=(1+npar1/ngrad0)/(1-(npar1+2)/ngrad0),
+                  "None"=log(ngrad0)-log(ngrad0-npar1),
                   log(ngrad0)*npar1/ngrad0)
   penIC2 <- switch(msc,"AIC"=2*npar2/ngrad0,"BIC"=log(ngrad0)*npar2/ngrad0,
                   "AICC"=(1+npar2/ngrad0)/(1-(npar2+2)/ngrad0),
+                  "None"=log(ngrad0)-log(ngrad0-npar2),
                   log(ngrad0)*npar2/ngrad0)
      krit1 <- log(sigma1[where])+penIC1[z1$order[where]+1]
      krit2 <- log(sigma2[where])+penIC2[z2$order[where]+1]
